#!/usr/local/bin/python
# EVG - ECLiPt Virtual Gallery Creator
# (c) 1999 by Emanuel Pirker <epirker@edu.uni-klu.ac.at>
# Distributed under the GNU GPL. 
# See http://www.gnu.org/copyleft/gpl.html#SEC1 for the full licence

import os, sys, signal, string
from eclipthtml import *

# ----------------------------------------------------------------------
# Constants

CONVERTCMD = '/usr/local/bin/convert'
VIEWERCMD = '/usr/X11R6/bin/xv'
THUMBNAILDIR = 'thumbnails'
THUMBPERCENT = '20%'
SERDBFILE = "serdb"
PICDBFILE = "picdb"
INDEXINFO = "index.html"
INDEXTHUMB = "index_thumb.html"
INDEXLIST = "index_list.html"
ROXENALBUM = "index_roxen.html"
UPLINK = "../"
IMAGEEXT = ['jpg', 'gif', 'png', 'tif']

# ----------------------------------------------------------------------
# Web-content generating classes

class EVGWeb:

    def LoadEVGDefaults(self):
	self.bgimage = None           # Background Image for WebPages
	self.text = "#ffffff"         # Default Text Color for WebPages
	self.bgcolor = "#000000"      # Default Background Color for WebPages
	self.link = "#ccccff"         # Default color for links
	self.alink = "#ff0000"        # Default color for activated links
	self.vlink = "#acacee"        # Default color for visited links
	self.headercolor = "#ccccff"  # Default color for headers
	self.errorcolor = "#cc0000"   # Default color for errors, warnings etc
	self.tableheadfg = "#eeeeee"  # Default table header forground color
	self.tableheadbg = "#222244"  # Default table header background color 
	self.tablebodyfg = "#ffffff"  # Default table foreground for the body
	self.tablebodybg = "#000000"  # Default table background for the body
        self.titlefg = "#eeeeee"      # Def foreground color of mirror title
        self.titlebg = "#222244"      # Def background color of mirror title
	self.border = False           # Default table border
	self.headerfile = ""          # Include file as header
	self.footerfile = ""          # Include file as footer


    def CreateNavBar( self, elems ):
	code = CenterHead() + '[ '

	for i in range( len(elems) ):
	    text, target = elems[i]
	    if target == "None.html":
		code = code + Font(text, color = self.text)
	    else:
		code = code + Link(text, target)
	    if i < ( len(elems) - 1 ):
		code = code + Font(' | ', color = self.text)

	code = code + " ] " + CenterTail() + Spacer("vertical", 20)

	return code

    def CreateStdHeader( self ):
	head = Head("Photo Gallery", "A:link { text-decoration: none; }\nA:active { text-decoration: none; } \nA:visited { text-decoration: none; } \ntable,tr,td,body,p,th,a,center { font-family:Helvetica; }")
	bodystart = BodyStart( self.text, self.bgcolor, self.link, 
			       self.alink, self.vlink, self.bgimage)
	return '<html>\n' + head + bodystart

    def CreateStdFooter( self ):
	copyright = ''
	endtags = '</body>\n</html>'
	return copyright + endtags

    def CreateWebPage (self, filename, content ):
	f = open( filename, 'w' )
	f.write( self.CreateStdHeader() + content + self.CreateStdFooter() )
	f.close()

class ImagePages(EVGWeb):

    def __init__(self,picdb):
	self.LoadEVGDefaults()
	self.picdb = picdb

    def CreateFile(self, filename, descr, first, prev, up, next, last):
	NavBarList = [ ( 'First', "%s.html" % first ), 
		       ( 'Prev', "%s.html" % prev ),  
		       ( 'Up', up ), ('Next', "%s.html" % next ), 
		       ( 'Last', "%s.html" % last ) ]

	navbar = self.CreateNavBar( NavBarList ) 
	img = Center( Image( filename, "%s" % descr, "border=0" ) 
		      + P() + descr + P() )

	self.CreateWebPage('%s.html' % filename, navbar + img + navbar)

    def GenerateImagePages( self ):
	# Generates a Web page for each file in picdb, nicely linked with 
	# First/Last/Prev/Next buttons

	uplink = 'index.html'

	i = 0  
	keys = self.picdb.keys()
	keys.sort()

	for pic in keys:
	    if i == 0:
		self.CreateFile( pic, self.picdb[pic], None, None, uplink, 
				 keys[i+1], keys[-1])
	    elif i == len(self.picdb) - 1:
		self.CreateFile( pic, self.picdb[pic], keys[0], keys[i-1], 
				 uplink, None, None)
	    else:
		self.CreateFile( pic, self.picdb[pic], keys[0], keys[i-1],
				 uplink, keys[i+1], keys[-1])

	    i = i + 1


class IndexPage( EVGWeb ):

    def __init__( self, picdb, serdb, uplink ):
	self.LoadEVGDefaults()
	self.picdb = picdb
	self.serdb = serdb
	self.up = uplink
	self.keys = self.picdb.keys()
	self.keys.sort()

    def CreateNavBar( self ):
	return EVGWeb.CreateNavBar( self, [ ('Info',INDEXINFO), 
					 ('Images', "%s.html" % self.keys[0]), 
					 ('List', INDEXLIST), 
					 ('Thumbnails', INDEXTHUMB), 
					 ('Up', self.up) ] )

    def CreateStdHeader(self):
	inheritedHeader = EVGWeb.CreateStdHeader(self)

	title = "<h1>" + self.serdb[ 'Name' ] + "</h1>"
	header = self.CreateNavBar() + Center(title) + Spacer( "vertical", 15 )

	return inheritedHeader + header

    def CreateThumbIndex( self ):
	keys = self.picdb.keys()
	keys.sort()
	content = ""
	for i in keys:
	    content = content + Link( Image( "%s/%s" % (THUMBNAILDIR, i), 
					     self.picdb[ i ], 
					     'border=1'), "%s.html" % i ) 

	self.CreateWebPage( INDEXTHUMB, content )

    def CreateRoxenAlbum( self ):
	content = '<album title="%s" nails="%s">\n' % ( 
	    self.serdb[ 'Name' ], THUMBNAILDIR )

	keys = self.picdb.keys()
	keys.sort()
	for i in keys:
	    content = content + '<photo file="%s">%s</photo>\n' % (
		i, self.picdb[i] )

	content = content + '</album>\n\n'

	self.CreateWebPage( ROXENALBUM, content )

    def CreateListIndex( self ):
	keys = self.picdb.keys()
	keys.sort()

	trows = []
	for i in keys:
	    image = Link( Image( "%s/%s" % (THUMBNAILDIR, i), 
				 i, "border=1" ), "%s.html" % i )
	    trows.append( TableRow( [ image, self.picdb[i] ] ) )

	self.CreateWebPage( INDEXLIST, Table( trows, border = 2 ) )


    def CreateIndex( self ):
	trows = []

	if self.serdb['Date'] != '':
	    trows = [ TableRow( [ 'Date:', self.serdb['Date'] ] ) ]

	if self.serdb['Description'] != '':
	    trows.append( TableRow( [ 'Description:', 
				      self.serdb[ 'Description' ] ] ) )

	if self.serdb['Photographer'] != '':
	    trows.append( TableRow( [ 'Photographer', 
				      self.serdb['Photographer' ] ] ) )

	if self.serdb['Region'] != '':
	    trows.append( TableRow( [ 'Region', self.serdb[ 'Region' ] ] ) )
	    
	if self.serdb['Locations'] != '':
	    trows.append( TableRow( [ 'Locations' , 
				      self.serdb['Locations'] ] ) )

	table = Table(trows, border = 2)
	
	if self.serdb['MapImage'] != '':
	    help = "<p><b>Click on the image below to start viewing the photos or on the top navigation bar to view a photo index...</b><p>"
	    img = Link( Image( self.serdb[ 'MapImage' ], 
			       'Map Image %s' % self.serdb[ 'MapImage' ], 
			       "border=0" ), "%s.html" % self.keys[0] )

	else:
	    help = "<p><b>Click on the top navigation bar to begin viewing images or see a thumbnail index...</b><p>"
	    img = ""

	self.CreateWebPage( INDEXINFO, table + help + img )

# ----------------------------------------------------------------------------
# Procedural code

def ReadOrCreateDatabase():
    # If a database is here, use it. If not, let the user create it 
    # interactively. Database means image comments (picdb), photo series 
    # location, photographer, date (serdb)

    if not os.path.exists(PICDBFILE):
	picdb = {}
	files = sys.argv[1:]
	if len(files) == 0:
	    print "You didn't provide any image file names on the command line."
	    print "I will use all pictures %s in the current directory!" % IMAGEEXT
	    allfiles = os.listdir(os.getcwd())
	    for i in allfiles:
		path,ext = os.path.splitext(i)
		if string.lower(ext)[1:] in IMAGEEXT:
		    files.append(i)
	    print files
	for i in files:
	    vpid = os.fork()
	    if vpid == 0:
		os.execl( VIEWERCMD, VIEWERCMD, i )
	    print "Descr of %s:" % i
	    picdb[ i ] = raw_input()
	    os.kill( vpid, signal.SIGTERM )
	    os.waitpid( vpid, 0 )
	file = open( PICDBFILE, "w" )
	file.write( repr( picdb ) )
	file.close()

    if not os.path.exists(SERDBFILE):
	serdb = {}
	print "Name of the photo series:"
	serdb['Name'] = raw_input()
	while serdb['Name'] == '':
	    print "You must enter a name... What else should appear as title?"
	    serdb['Name'] = raw_input()
	print "Description (What is shown on this image/photo series):"
	serdb['Description'] = raw_input()
	print "Date or date range when these photos were taken:"
	serdb['Date'] = raw_input()
	print "Photographer:"
	serdb['Photographer'] = raw_input()
	print "World Region (e.g. California, USA):"
	serdb['Region'] = raw_input()
	print "Intro Image (one 'typical' image you want to have on the main page):"
	serdb['MapImage'] = raw_input()
	print "Detailed Locations (e.g. L.A., San Francisco):"
	serdb['Locations'] = raw_input()

	file = open( SERDBFILE, "w" )
	file.write( repr(serdb) )
	file.close()

    f = open( PICDBFILE, 'r')
    dbstring = ""
    line = f.readline()
    while line<>'':
	dbstring = dbstring + line
	line = f.readline()
    f.close()

    picdb = eval(dbstring)

    f = open( SERDBFILE, 'r')
    dbstring = ""
    line = f.readline()
    while line<>'':
	dbstring = dbstring + line
	line = f.readline()
    f.close()

    serdb = eval(dbstring)

    return ( picdb, serdb )

def main():
    # Read Databases (image descriptions, etc)
    # If they don't exist, let the user create it interactively
    ( picdb, serdb ) = ReadOrCreateDatabase()

    # Generate image pages; one Web page for each image
    imgPages = ImagePages( picdb )
    imgPages.GenerateImagePages()

    # Generate thumbnails by using ImageMagick's convert program
    if not os.path.exists( THUMBNAILDIR ):
	os.mkdir( THUMBNAILDIR )
	files = sys.argv[1:]
	for filename in files:
	    os.system( "%s -geometry %sx%s %s %s" % 
		       ( CONVERTCMD, THUMBPERCENT, THUMBPERCENT, filename, 
			 THUMBNAILDIR + '/' + filename ) )

    # Generate various index pages; Thumbnail-Index, List-Index, Roxen-Album
    # and Main-Info-Page
    indexGen = IndexPage( picdb, serdb, UPLINK )
    indexGen.CreateThumbIndex()
    indexGen.CreateListIndex()
    indexGen.CreateRoxenAlbum()
    indexGen.CreateIndex()

main()
